//------------------------------------------------------------------------------
//
// app.h
//
//------------------------------------------------------------------------------
#include "common.h"

//internal
HRESULT InitDirectX();
HRESULT TermDirectX();

HWND      hWnd          = NULL;
HINSTANCE hInst         = NULL;
DWORD     dwWidth       = 0;
DWORD     dwHeight      = 0;
BOOL      bIsWindow     = FALSE;
char      szAppName[MAX_PATH + 1];

WORD      wMouseX       = 0;
WORD      wMouseY       = 0;
BOOL      bMouseLButton = FALSE;
BOOL      bMouseRButton = FALSE;
BOOL      bMouseMButton = FALSE;

POINT GetMousePos() {
  POINT p = {0, 0};
  if(hWnd) {
    GetCursorPos(&p);
    ScreenToClient(hWnd, &p);
  }
  return p;
}

DWORD GetWidth() {
  return dwWidth;
}

DWORD GetHeight() {
  return dwHeight;
}

BOOL GetIsWindow() {
  return bIsWindow;
}

HWND GetWindowHandle() {
  return hWnd;
}

HINSTANCE GetInstanceHandle() {
  return hInst;
}

static LRESULT WINAPI SimpleMsgProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam ) {
  switch( msg ) {
    //need 
    case WM_SYSCOMMAND:
      switch( ( wParam & 0xFFF0 ) ) {
        case SC_MONITORPOWER:
        case SC_SCREENSAVE:
          return 0; // DefWindowProc not call
          break;
      }
      break;

    case WM_CLOSE:
    case WM_DESTROY:
      ::PostQuitMessage( 0 );
      return 0;
      break;

    case WM_KEYDOWN:
      if( wParam == VK_ESCAPE ) {
        ::PostQuitMessage( 0 );
      }
      break;

    case WM_LBUTTONDOWN:
      bMouseLButton = TRUE;
      break;
      
    case WM_RBUTTONDOWN:
      bMouseRButton = TRUE;
      break;
      
    case WM_MBUTTONDOWN:
      bMouseMButton = TRUE;
      break;

    case WM_LBUTTONUP:
      bMouseLButton = FALSE;
      break;
      
    case WM_RBUTTONUP:
      bMouseRButton = FALSE;
      break;
      
    case WM_MBUTTONUP:
      bMouseMButton = FALSE;
      break;
    
     
    case WM_IME_SETCONTEXT:
      lParam &= ~ISC_SHOWUIALL;
      break;
  }
  return ::DefWindowProc( hWnd, msg, wParam, lParam );
}

HRESULT Term() {
  TermDirectX();
  
  if(hInst) {
    timeEndPeriod(1);
    ::UnregisterClass(szAppName, hInst);
    hInst = NULL;
    hWnd  = NULL;
  }

  return S_OK;
}

//entry point
HRESULT Init(LPSTR szName, DWORD w, DWORD h, BOOL bWindow) {
  HRESULT   hRet        = E_FAIL;
  DWORD     style       = 0;
  DWORD     dwExStyle   = 0;
  POINT     point;
  RECT      rc;

  hInst     = ::GetModuleHandle(NULL);
  dwWidth   = w;
  dwHeight  = h;
  bIsWindow = bWindow;
  wsprintf(szAppName, "%s", szName);
  
  WNDCLASSEX twc = {
    sizeof( WNDCLASSEX ), CS_CLASSDC | CS_VREDRAW | CS_HREDRAW, SimpleMsgProc, 0L, 0L,
    hInst, ::LoadIcon( hInst, TEXT( "MAINICO" ) ), ::LoadCursor( NULL , IDC_ARROW ), 
    ( HBRUSH )::GetStockObject( BLACK_BRUSH ), NULL, szName, NULL
  };
  
  if ( !::RegisterClassEx( &twc ) ) {
    Term();
    return hRet;
  }
  
  if(!bWindow) {
    dwExStyle = WS_EX_APPWINDOW;
    style     = WS_POPUP;
  } else {
    dwExStyle = WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;
    style     = WS_OVERLAPPEDWINDOW;
  }
  
  ::SetRect( &rc, 0, 0, dwWidth, dwHeight );
  point.x     = ::GetSystemMetrics( SM_CXSCREEN );
  point.y     = ::GetSystemMetrics( SM_CYSCREEN );
  ::AdjustWindowRectEx( &rc, style, FALSE, dwExStyle);

  hWnd = ::CreateWindowEx(
    dwExStyle, szAppName, szAppName, style,
    ( point.x - dwWidth ) / 2, ( point.y - dwHeight ) / 2,
    rc.right - rc.left, rc.bottom - rc.top,
    NULL, NULL, hInst, NULL );
  
  if(!hWnd) {
    Term();
    return hRet;
  }
  
  //Show Window
  ::ShowWindow(hWnd, SW_SHOW);
  ::UpdateWindow(hWnd);
  ::timeBeginPeriod(1);

  return InitDirectX();
}


BOOL DoEvent() {
  MSG   msg;
  BOOL  bActive = TRUE;
  if(hWnd) {
    while(::PeekMessage(&msg , 0 , 0 , 0 , PM_REMOVE)) {
      if(msg.message == WM_QUIT) {
        bActive = FALSE;
        break;
      } else {
        ::TranslateMessage( &msg );
        ::DispatchMessage( &msg );
      }
    }
    Sleep(0);
  }
  return bActive;
}

