//------------------------------------------------------------------------------
//
// render.cpp
//
//------------------------------------------------------------------------------
#include "common.h"

//enum
enum {
  LostSleepTime = 250,
  MaxLostRepeat = 30,
};

//DirectX
static LPDIRECT3D9              lpD3D       = NULL;
static LPDIRECT3DDEVICE9        lpD3DDevice = NULL;
static D3DPRESENT_PARAMETERS    d3dpp;
static D3DADAPTER_IDENTIFIER9   d3daid;
static D3DDISPLAYMODE           d3ddm;

//Ƀ_p̐FX


HRESULT TermResource() {
  return S_OK;
}

HRESULT TermDirectX() {
  
  TermResource();
  
  RELEASE(lpD3DDevice);
  RELEASE(lpD3D);
  return S_OK;
}


LPDIRECT3DDEVICE9 GetDevice() {
  return lpD3DDevice;
}

LPSTR DisplayDriverName() {
  return d3daid.Driver;
}

LPSTR DisplayDescriptionName() {
  return d3daid.Description;
}

HRESULT BeginScene() {
  HRESULT hRet = E_FAIL;
  if(lpD3DDevice) {
    hRet = lpD3DDevice->BeginScene();
  }
  return hRet;
}

static HRESULT OnLost() {
  HRESULT hRet = E_FAIL;
  hRet = S_OK;
  return hRet;
}

static HRESULT OnRestore() {
  HRESULT hRet = E_FAIL;
  hRet = S_OK;
  return hRet;
}

static HRESULT HandlePresentParam(HRESULT hError, int nRecursive) {
  HRESULT hRet = E_FAIL;
  switch(hError) {
  case D3DERR_DRIVERINTERNALERROR:
    PostQuitMessage(0);
    break;

  case D3DERR_DEVICELOST: {
    if(nRecursive == 0) {
      hRet = HandlePresentParam(lpD3DDevice->TestCooperativeLevel(), 1);
      break;
    } else {
      ::Sleep(LostSleepTime);
      break;
    }
    break;
  }

  case D3DERR_DEVICENOTRESET: {
    DWORD nRepeat = 0;
    while( OnLost() ) {
      nRepeat++;
      if(nRepeat > MaxLostRepeat) {
        ::PostQuitMessage(0);
        hRet = E_FAIL;
        return hRet;
      }
    }

    hRet = lpD3DDevice->Reset(&d3dpp);
    if(hRet != D3D_OK) {
      ::PostQuitMessage(0);
      break;
    }

    hRet = OnRestore();
    if(hRet != D3D_OK) {
      ::PostQuitMessage(0);
      break;
    }
    break;
  }
  default:
    break;
  }
  return hRet;
}

HRESULT EndScene() {
  HRESULT hRet = E_FAIL;
  if(lpD3DDevice) {
    hRet = lpD3DDevice->EndScene();
  }
  return hRet;
}


HRESULT Flip() {
  HRESULT hRet = E_FAIL;
  if(lpD3DDevice) {
    hRet = lpD3DDevice->Present(NULL, NULL, NULL, NULL);
    if(hRet != D3D_OK) {
      hRet = HandlePresentParam(hRet, 0);
    }
  }
  return hRet;
}

HRESULT Clear(D3DCOLOR color) {
  HRESULT hRet = E_FAIL;
  if(lpD3DDevice) {
    if(d3dpp.EnableAutoDepthStencil) {
      hRet = lpD3DDevice->Clear(0, NULL, D3DCLEAR_TARGET | D3DCLEAR_ZBUFFER, color, 1.0f, 0);
    } else {
      hRet = lpD3DDevice->Clear(0, NULL, D3DCLEAR_TARGET , color, 1.0f, 0);
    }
  }
  return hRet;
}

HRESULT StartScene(D3DCOLOR color) {
  HRESULT hRet = E_FAIL;
  Clear(color);
  hRet = BeginScene();
  return hRet;
}

HRESULT CreateNormalResource() {
  HRESULT hRet = E_FAIL;

  //Create DirectX Device
  ZeroMemory(&d3dpp, sizeof(d3dpp));
  d3dpp.Windowed                = GetIsWindow();
  d3dpp.BackBufferCount         = 1;
  d3dpp.SwapEffect              = D3DSWAPEFFECT_DISCARD;
  d3dpp.BackBufferFormat        = d3ddm.Format;
  //d3dpp.BackBufferFormat        = D3DFMT_UNKNOWN;
  d3dpp.MultiSampleType         = D3DMULTISAMPLE_4_SAMPLES;
  
  d3dpp.PresentationInterval    = D3DPRESENT_INTERVAL_DEFAULT;
  d3dpp.EnableAutoDepthStencil  = TRUE;
  d3dpp.AutoDepthStencilFormat  = D3DFMT_D16;
  d3dpp.BackBufferWidth         = GetWidth();
  d3dpp.BackBufferHeight        = GetHeight();
  
  
  //HAL + MultiSample
  hRet = lpD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, GetWindowHandle(), D3DCREATE_HARDWARE_VERTEXPROCESSING, &d3dpp, &lpD3DDevice);
  if(hRet == D3D_OK) {
    return hRet;
  }
  
  
  d3dpp.MultiSampleType = D3DMULTISAMPLE_NONMASKABLE;
  
  //HAL + NonMultiSample
  hRet = lpD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, GetWindowHandle(), D3DCREATE_HARDWARE_VERTEXPROCESSING, &d3dpp, &lpD3DDevice);
  if(hRet == D3D_OK) {
    return hRet;
  }
 
  //HEL
  hRet = lpD3D->CreateDevice(D3DADAPTER_DEFAULT, D3DDEVTYPE_HAL, GetWindowHandle(), D3DCREATE_SOFTWARE_VERTEXPROCESSING, &d3dpp, &lpD3DDevice);
  if(hRet == D3D_OK) {
    return hRet;
  }

  return E_FAIL;
}

HRESULT InitDirectX() {
  HRESULT hRet = E_FAIL;
  
  //Initialize Direct Graphics
  lpD3D = Direct3DCreate9(D3D_SDK_VERSION);
  if(!lpD3D) {
    return hRet;
  }
  
  hRet = lpD3D->GetAdapterIdentifier(D3DADAPTER_DEFAULT, 0, &d3daid);
  if(hRet != D3D_OK) {
    return hRet;
  }
  
  hRet = lpD3D->GetAdapterDisplayMode(D3DADAPTER_DEFAULT, &d3ddm);
  if(hRet != D3D_OK) {
    return hRet;
  }
  
  return CreateNormalResource();
}
